function cellValue(table, r, c) {

    var cell = table[r][c];

    if (cell.resolved === false) {
        throw new Error("not yet calculated");
    }

    var text = cell.value;

    if (text === '') {
        return '';
    }

    text = text.trim();

    if (isFinite(text)) {
        return Number(text);
    }

    // %対応
    if (text[text.length - 1] === '%') {
        return Number(text.substring(0, text.length - 1) / 100);
    }

    // カンマ対応
    var v = text.replace(/,/g, '');
    if (isFinite(v)) {
        return Number(v);
    }

    return String(text);
}

function getRangeValues(table, startCellCoord, endCellCoord) {
    var fragment = [];

    for (var row = startCellCoord.row.index; row <= endCellCoord.row.index; row++) {
        var colFragment = [];

        for (var col = startCellCoord.column.index; col <= endCellCoord.column.index; col++) {
            colFragment.push(cellValue(table, row, col));
        }
        fragment.push(colFragment);
    }

    return fragment;
}

function createParser(defaultTable, tables) {
    var p = new formulaParser.Parser();

    p.on('callCellValue', function(cellCoord, done) {
        var t = (cellCoord.sheet === undefined) ? tables[defaultTable] : tables[cellCoord.sheet - 1];
        done(cellValue(t, cellCoord.row.index, cellCoord.column.index));
    });

    p.on('callRangeValue', function(startCellCoord, endCellCoord, done) {
        var t = (startCellCoord.sheet === undefined) ? tables[defaultTable] : tables[startCellCoord.sheet - 1];
        var fragment = getRangeValues(t, startCellCoord, endCellCoord);

        if (fragment) {
            done(fragment);
        }
    });

    return p;
}

function doFormat(format, result) {
    if (format !== null && format.length > 0) {
        var dataFormatter = new DataFormatter();
        dataFormatter.locale.thousandSeparator = ',';
        var r = dataFormatter.format(result, 'Number', format);
        return r.value;
    }
    return result;
}

function evaluate(cells, formulas) {

    // create parser for each formula
    var parsers = {};
    formulas.forEach(function(cell) {
        parsers[cell.macroId] = createParser(cell.tableNumber, cells);
    });

    // calculation loop
    var results = new java.util.HashMap();
    var errorCnt = 0;
    while (formulas.length !== 0 && errorCnt < formulas.length) {
        var e = formulas.remove(0);
        var result = parsers[e.macroId].parse(e.value);

        if (result.error === null) {
            e.value = result.result;
            e.resolved = true;
            results.put(e.macroId, doFormat(e.excelFormat, result.result));
            errorCnt = 0;
        } else {
            results.put(e.macroId, result.error);
            formulas.add(e);
            ++errorCnt;
        }
    }

    return results;
}
