/**
 * MCE用のプラグイン
 */
define('com/excelimport/plugins', ['ajs', 'jquery'], function (AJS, $){
    "use strict";

    if (document.body.querySelector("#cipe-editor") !== null) {
        return;
    }

    var workbook;

    /**
     * ファイルが選択されたときの処理
     *
     *   - Excelファイルを解析する
     *   - シート一覧をシート選択肢に加え、UI可視化
     *
     * @param e
     */
    function handleFileSelect(e) {
        var files = e.target.files,file;
        if (!files || files.length == 0) {
            return;
        }

        var file = files[0];
        var fileReader = new FileReader();
        fileReader.onload = function (e) {
            // pre-process data
            var binary = "";
            var bytes = new Uint8Array(e.target.result);
            var length = bytes.byteLength;
            for (var i = 0; i < length; i++) {
                binary += String.fromCharCode(bytes[i]);
            }
            // call 'xlsx' to read the file
            workbook  = XLSX.read(binary, {type: 'binary', cellDates:true, cellStyles:true});

            var $sheetSelect = $("#select-sheet");
            $sheetSelect.empty();

            workbook.SheetNames.forEach(function(e){

                var option = Confluence.Templates.QCM.option({value: e, label: e});
                $sheetSelect.append(option);

            });

            $('.qcm-step2').show();
            $("#dialog-submit-button").prop("disabled", false);
            $sheetSelect.auiSelect2();

        };
        fileReader.readAsArrayBuffer(file);

    }

    /**
     * Sumbit時の処理
     *
     *   - シートをhtmlに変換しテーブル部分を取り出す
     *   - テーブルの各要素にConfluence用のスタイルクラスを適用する
     *   - 式をマクロに置き換える
     *   - 処理が終わったらダイアログを閉じる
     *
     * @param e
     */
    function onHide(e) {
        e.preventDefault();

        var sheetName = $("#select-sheet").val();
        var worksheet = workbook.Sheets[sheetName];

        var $html = $(XLSX.utils.sheet_to_html(worksheet));
        var $table = $($html[2]);

        $table.addClass('confluenceTable');
        $table.find('th').addClass('confluenceTh');
        $table.find('td').addClass('confluenceTd');

        tinymce.activeEditor.execCommand('mceInsertContent', false, $table.prop('outerHTML'));

        var $iframe  = $('#wysiwygTextarea_ifr');
        var ifrmDoc = $iframe[0].contentWindow.document;

        // 関数があったら該当セルにマクロを挿入
        Object.keys(worksheet).forEach(function(e){
            if (worksheet[e].f) {
                // 式がある場合

                var $ele  = $('#sjs-' + e, ifrmDoc);
                var $textNode = $ele
                    .contents()
                    .filter(function() {
                        return this.nodeType === 3; //Node.TEXT_NODE
                    });
                tinyMCE.activeEditor.selection.select($textNode.get(0));

                var parameters = {};
                if (worksheet[e].t === 'n') {
                    parameters['predefined-format'] = 'General Number';
                    parameters['horizontal-alignment'] = 'Right';
                }

                tinymce.confluence.MacroUtils.insertMacro({
                    contentId: AJS.Meta.get('content-id'),
                    macro: {
                        body: worksheet[e].f,
                        name: "qron-calc-macro",
                        params: parameters
                    }
                });
            } else {
                // 定数値のセルの場合
                if (worksheet[e].t === 'n') {
                    var $ele  = $('#sjs-' + e, ifrmDoc);
                    $ele.attr('style', 'text-align: right');
                }
            }
        });

        AJS.dialog2('#import-dialog').hide();
    }

    // プラグインのコマンド
    function mceQCMExcelImport() {

        // dialog要素が無かったら、要素を挿入
        if ($("#import-dialog").length === 0) {
            $('body').append(Confluence.Templates.QCM.importDialog({}));
            document.getElementById('files').addEventListener('change', handleFileSelect, false);
        }

        // dialogを開く
        $('.qcm-step2').hide();
        $("#dialog-submit-button").prop("disabled", true);
        $('#files').val('');
        AJS.dialog2('#import-dialog').show();

        AJS.$('#dialog-submit-button').unbind( 'click' ).click(onHide);
    }

    return {
        init : function(ed) {
            // グループを登録
            $("#rte-toolbar .toolbar-primary").append($("<ul class='aui-buttons'></ul>")
                .addClass("rte-toolbar-group-qcm"));

            ed.addCommand('mceQCMExcelImport',　mceQCMExcelImport);

            // Register button in a new group
            ed.addButton('qcm-excelimport-button', {
                title: '',
                tooltip: 'Excel Importer',
                cmd: 'mceQCMExcelImport',
                className: 'qcm-excelimport',
                icon: 'aui-icon aui-icon-small aui-iconfont-file-xls',
                locationGroup: 'rte-toolbar-group-qcm',
                weight: 0
            });
        },
        getInfo : function() {
            return {
                longname : 'Qron Calc Macro - Excel Sheet Importer',
                author : 'Kenputer',
                authorurl : 'http://kenputer.netlify.com',
                version : '1.1'
            };
        }
    }
});

/**
 * プラグインをMCEに登録
 */
require('confluence/module-exporter')
    .safeRequire('com/excelimport/plugins', function(ExcelImportPlugin) {

        return;

        if (document.body.querySelector("#cipe-editor") !== null) {
            return;
        }

        var tinymce = require('tinymce');

        tinymce.create('tinymce.plugins.ExcelImportPlugin', ExcelImportPlugin);

        // Register plugin
        tinymce.PluginManager.add('excelimport', tinymce.plugins.ExcelImportPlugin);

        require('confluence-editor/loader/tinymce-bootstrap').addTinyMcePluginInit(function(settings) {
            settings.plugins += ",excelimport";
        });
    });

