Confluence.QcalcEditDialogs = Confluence.QcalcEditDialogs || (function($) {

    var DIALOG_MAX_HEIGHT = 200;
    var DIALOG_WIDTH = 300;
    var editDialog;
    var defaultDialogOptions = {
        hideDelay: null,
        width : DIALOG_WIDTH,
        maxHeight: DIALOG_MAX_HEIGHT
    };
    var previousFormula;

    function showEditDialog(selectionObject) {
        editDialog && editDialog.remove();

        var $target = $(selectionObject.containingElement);
        var encodedFormula = $target.attr('qcm-formula');

        if (encodedFormula === undefined) {
            return;
        }

        var macroId = $target.attr('qcm-macro-id');
        previousFormula = decodeURIComponent(encodedFormula.replace(/[+]/g, ' '));

        var displayFn = function(content, trigger, showPopup) {
            $(content).html(Confluence.QcalcEditDialogs.Templates.createDialogContent(
                {
                    highlightText: previousFormula
                }
            ));

            plantFormulaEditor(content, $target, macroId);

            showPopup();
            return false;
        };
        editDialog = _openDialog(selectionObject, 'qcalc-edit-dialog', defaultDialogOptions, displayFn);
    }

    /**
     * 初期状態では、式の表示のみ。
     * - 式をクリックすると編集画面に切り替わる
     * @param content
     * @param $targetEl
     * @param macroId
     */
    function plantFormulaEditor(content, $targetEl, macroId) {

        var $quickview = $(content).find('.qcm-formula-quickview');
        var $quickedit = $(content).find('.qcm-formula-quickedit');

        $quickview.show();
        $quickedit.hide();
        $quickedit.focus(function(){
           $(this).select();
        });
        $quickview.click(function(){
            $quickview.hide();
            $quickedit.show();
            $quickedit.focus();
        });

        $quickedit.keydown(function(e) {
            if ((e.which && e.which === 13) || (e.keyCode && e.keyCode === 13)) {
                var newFormula = $quickedit.val();
                if (newFormula !== previousFormula) {
                    updateFormula(newFormula, macroId);
                    $targetEl.attr('qcm-formula', encodeURIComponent(newFormula));
                } else {
                    editDialog && editDialog.hide();
                }
                return false;
            } else {
                return true;
            }
        });
    }

    /**
     * 式を保存し、処理終了後、ダイアログを閉じる
     * @param newFormula
     */
    function updateFormula(newFormula, macroId) {
        AJS.dim();

        $.ajax({
            url: AJS.contextPath() + '/rest/qcmquickedit/latest/formula',
            type: 'PUT',
            dataType: 'json',
            contentType: 'application/json',
            data: JSON.stringify({
                pageId: AJS.Meta.get("page-id"),
                macroId: macroId,
                formula: newFormula
            }),
            success: function(resp) {
               editDialog && editDialog.hide();
               AJS.undim();
               window.XQreload();
            },
            error: function(x) {
                editDialog && editDialog.hide();
                AJS.undim();
                AJS.flag({type: 'error', title: 'Quick edit', body: x.responseText});
            }
        });
    }

    function _openDialog(selectionObject, id, options, displayFn) {
        var $target = $("<div>");
        _appendDialogTarget(selectionObject.area.average, $target);
        options.hideCallback = function() {
            $target.remove(); // clean up dialog target element when hiding the dialog
            Confluence.QronCalcMacroTag.hide();
        };

        var dialog = Confluence.ScrollingInlineDialog($target, id, displayFn, options);
        dialog.show();
        return dialog;
    }

    function _appendDialogTarget(targetDimensions, $target) {
        Confluence.DocThemeUtils.appendAbsolutePositionedElement($target);
        $target.css({
            top: targetDimensions.top,
            height: targetDimensions.height,
            left: targetDimensions.left,
            width: targetDimensions.width,
            "z-index": -9999,
            position: 'absolute'
        });
    }

    return {
        showEditDialog: showEditDialog
    }
})(AJS.$);
